'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

/* eslint space-before-function-paren:0 */
exports.default = {
  getMousePosition: function getMousePosition(event) {
    return {
      x: event.pageX - (window.scrollX || window.pageXOffset),
      y: event.pageY - (window.scrollY || window.pageYOffset)
    };
  },
  getTouchPosition: function getTouchPosition(event) {
    return {
      x: event.touches[0].pageX - (window.scrollX || window.pageXOffset),
      y: event.touches[0].pageY - (window.scrollY || window.pageYOffset)
    };
  },
  pauseEvent: function pauseEvent(event) {
    event.stopPropagation();
    event.preventDefault();
  },
  addEventsToDocument: function addEventsToDocument(eventMap) {
    for (var key in eventMap) {
      if (Object.prototype.hasOwnProperty.call(eventMap, key)) {
        document.addEventListener(key, eventMap[key], false);
      }
    }
  },
  removeEventsFromDocument: function removeEventsFromDocument(eventMap) {
    for (var key in eventMap) {
      if (Object.prototype.hasOwnProperty.call(eventMap, key)) {
        document.removeEventListener(key, eventMap[key], false);
      }
    }
  },
  targetIsDescendant: function targetIsDescendant(event, parent) {
    var clickedNode = event.currentTarget;

    // if the node exists,
    // the node is not the given parent,
    // and the node does not contain the parent,
    // then the node is a descendant of the parent
    if (clickedNode && clickedNode !== parent && !clickedNode.contains(parent)) {
      return true;
    }
    // otherwise it is not a descendant of the given parent
    return false;
  },
  addEventListenerOnTransitionEnded: function addEventListenerOnTransitionEnded(element, fn) {
    var eventName = transitionEventNamesFor(element);
    if (!eventName) return false;
    element.addEventListener(eventName, fn);
    return true;
  },
  removeEventListenerOnTransitionEnded: function removeEventListenerOnTransitionEnded(element, fn) {
    var eventName = transitionEventNamesFor(element);
    if (!eventName) return false;
    element.removeEventListener(eventName, fn);
    return true;
  }
};

// constants and helper functions /////////

// $FlowFixMe

var TRANSITIONS = {
  transition: 'transitionend',
  OTransition: 'oTransitionEnd',
  MozTransition: 'transitionend',
  WebkitTransition: 'webkitTransitionEnd'
};

function transitionEventNamesFor(element) {
  for (var _transition in TRANSITIONS) {
    if (element && Object.prototype.hasOwnProperty.call(element.style, _transition)) {
      return TRANSITIONS[_transition];
    }
  }
}
//# sourceMappingURL=events.js.map