'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _cloneDeep2 = require('lodash/cloneDeep');

var _cloneDeep3 = _interopRequireDefault(_cloneDeep2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var appendToProperty = function appendToProperty(dest, name, value) {
  dest[name] === '' ? dest[name] = value : dest[name] += ' ' + value;
};

var composeComponentStyles = function composeComponentStyles(componentStyles, componentTheme) {
  if (!componentTheme) return;
  for (var property in componentStyles) {
    if (Object.prototype.hasOwnProperty.call(componentStyles, property)) {
      if (Object.prototype.hasOwnProperty.call(componentTheme, property)) {
        appendToProperty(componentStyles, property, componentTheme[property]);
      }
    }
  }
};

// checks for the existence of a property on theme
// that matches the value of themeId (string)
// if the property exists, also checks the type of
// theme[themeId] to ensure it's an object
var addThemeId = function addThemeId(theme, themeId) {
  if (!(0, _isEmpty3.default)(theme) && themeId) {
    var themeIdExists = Object.prototype.hasOwnProperty.call(theme, themeId);
    var themeIdIsObj = _typeof(theme[themeId]) === 'object';

    return themeIdExists && themeIdIsObj ? theme : _defineProperty({}, themeId, theme);
  }

  return theme;
};

/**
 * Composes a base theme with the given overrides, which should
 * be provided in the same schema, defined by the theme API param.
 *
 * @param theme - The base theme to be composed with overrides
 * @param themeOverrides - The custom overrides for the base theme
 * @param themeAPI - The theme API schema that should be used for composition
 * @returns {{}} - The composed theme
 */

var composeTheme = function composeTheme() {
  var theme = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var themeOverrides = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var themeAPI = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  // Return theme if there are no overrides provided
  if ((0, _isEmpty3.default)(themeOverrides)) return theme;

  var composedTheme = (0, _cloneDeep3.default)(themeAPI);

  for (var componentId in themeAPI) {
    if (Object.prototype.hasOwnProperty.call(composedTheme, componentId)) {
      var componentStyles = composedTheme[componentId];
      composeComponentStyles(componentStyles, theme[componentId]);
      composeComponentStyles(componentStyles, themeOverrides[componentId]);
    }
  }
  return composedTheme;
};

exports.default = {
  addThemeId: addThemeId,
  composeTheme: composeTheme
};
//# sourceMappingURL=themes.js.map